@@ -34,32 +34,46 @@ static void report(int f)
 	extern int am_sender;
 	extern int am_daemon;
 	extern int do_stats;
+	extern int remote_version;
+	int send_stats;
 
 	if (am_daemon) {
 		log_exit(0, __FILE__, __LINE__);
 		if (f == -1 || !am_sender) return;
 	}
 
-	if (!verbose) return;
-
-	if (am_server && !am_sender) return;
-
-	if (am_server && am_sender) {
-		write_longint(f,stats.total_read);
-		write_longint(f,stats.total_written);
-		write_longint(f,stats.total_size);
+	send_stats = verbose || (remote_version >= 20);
+	if (am_server) {
+		if (am_sender && send_stats) {
+			int64 w;
+			/* store total_written in a temporary
+			    because write_longint changes it */
+			w = stats.total_written;
+			write_longint(f,stats.total_read);
+			write_longint(f,w);
+			write_longint(f,stats.total_size);
+		}
 		return;
 	}
-    
-	if (!am_sender) {
+
+	/* this is the client */
+	    
+	if (!am_sender && send_stats) {
 		int64 r;
 		stats.total_written = read_longint(f);
+		/* store total_read in a temporary, read_longint changes it */
 		r = read_longint(f);
 		stats.total_size = read_longint(f);
 		stats.total_read = r;
 	}
 
 	if (do_stats) {
+		if (!am_sender && !send_stats) {
+		    /* missing the bytes written by the generator */
+		    rprintf(FINFO, "\nCannot show stats as receiver because remote protocol version is less than 20\n");
+		    rprintf(FINFO, "Use --stats -v to show stats\n");
+		    return;
+		}
 		rprintf(FINFO,"\nNumber of files: %d\n", stats.num_files);
 		rprintf(FINFO,"Number of files transferred: %d\n", 
 		       stats.num_transferred_files);
@@ -78,13 +92,15 @@ static void report(int f)
 		       (double)stats.total_read);
 	}
 	
-	rprintf(FINFO,"wrote %.0f bytes  read %.0f bytes  %.2f bytes/sec\n",
-	       (double)stats.total_written,
-	       (double)stats.total_read,
-	       (stats.total_written+stats.total_read)/(0.5 + (t-starttime)));
-	rprintf(FINFO,"total size is %.0f  speedup is %.2f\n",
-	       (double)stats.total_size,
-	       (1.0*stats.total_size)/(stats.total_written+stats.total_read));
+	if (verbose || do_stats) {
+		rprintf(FINFO,"wrote %.0f bytes  read %.0f bytes  %.2f bytes/sec\n",
+		       (double)stats.total_written,
+		       (double)stats.total_read,
+		       (stats.total_written+stats.total_read)/(0.5 + (t-starttime)));
+		rprintf(FINFO,"total size is %.0f  speedup is %.2f\n",
+		       (double)stats.total_size,
+		       (1.0*stats.total_size)/(stats.total_written+stats.total_read));
+	}
 
 	fflush(stdout);
 	fflush(stderr);
@@ -171,7 +187,10 @@ static char *get_local_name(struct file_list *flist,char *name)
 
 	if (verbose > 2)
 		rprintf(FINFO,"get_local_name count=%d %s\n", 
-			flist->count, name);
+			flist->count, NS(name));
+
+	if (!name) 
+		return NULL;
 
 	if (do_stat(name,&st) == 0) {
 		if (S_ISDIR(st.st_mode)) {
@@ -192,9 +211,6 @@ static char *get_local_name(struct file_list *flist,char *name)
 	if (flist->count == 1)
 		return name;
 
-	if (!name) 
-		return NULL;
-
 	if (do_mkdir(name,0777 & ~orig_umask) != 0) {
 		rprintf(FERROR,"mkdir %s : %s (1)\n",name,strerror(errno));
 		exit_cleanup(RERR_FILEIO);
@@ -318,6 +334,7 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 	char *local_name=NULL;
 	char *dir = NULL;
 	extern int delete_mode;
+	extern int delete_excluded;
 	extern int am_daemon;
 
 	if (verbose > 2)
@@ -334,7 +351,7 @@ static void do_server_recv(int f_in, int f_out, int argc,char *argv[])
 		}    
 	}
 
-	if (delete_mode)
+	if (delete_mode && !delete_excluded)
 		recv_exclude_list(f_in);
 
 	flist = recv_file_list(f_in);
@@ -391,9 +408,10 @@ int client_run(int f_in, int f_out, int pid, int argc, char *argv[])
 	if (am_sender) {
 		extern int cvs_exclude;
 		extern int delete_mode;
+		extern int delete_excluded;
 		if (cvs_exclude)
 			add_cvs_excludes();
-		if (delete_mode) 
+		if (delete_mode && !delete_excluded) 
 			send_exclude_list(f_out);
 		flist = send_file_list(f_out,argc,argv);
 		if (verbose > 3) 
@@ -592,7 +610,7 @@ int main(int argc,char *argv[])
 	   carried across */
 	orig_umask = (int)umask(0);
 
-	if (!parse_arguments(argc, argv)) {
+	if (!parse_arguments(argc, argv, 1)) {
 		exit_cleanup(RERR_SYNTAX);
 	}
 
@@ -606,6 +624,12 @@ int main(int argc,char *argv[])
 	signal(SIGHUP,SIGNAL_CAST sig_int);
 	signal(SIGTERM,SIGNAL_CAST sig_int);
 
+	/* Initialize push_dir here because on some old systems getcwd
+	   (implemented by forking "pwd" and reading its output) doesn't
+	   work when there are other child processes.  Also, on all systems
+	   that implement getcwd that way "pwd" can't be found after chroot. */
+	push_dir(NULL,0);
+
 	if (am_daemon) {
 		return daemon_main();
 	}
